"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metric import Metric, MetricTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MetricsTypedDict(TypedDict):
    orders: MetricTypedDict
    r"""Information about a metric."""
    revenue: MetricTypedDict
    r"""Information about a metric."""
    net_revenue: MetricTypedDict
    r"""Information about a metric."""
    cumulative_revenue: MetricTypedDict
    r"""Information about a metric."""
    net_cumulative_revenue: MetricTypedDict
    r"""Information about a metric."""
    average_order_value: MetricTypedDict
    r"""Information about a metric."""
    net_average_order_value: MetricTypedDict
    r"""Information about a metric."""
    one_time_products: MetricTypedDict
    r"""Information about a metric."""
    one_time_products_revenue: MetricTypedDict
    r"""Information about a metric."""
    one_time_products_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    active_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    monthly_recurring_revenue: MetricTypedDict
    r"""Information about a metric."""
    committed_monthly_recurring_revenue: MetricTypedDict
    r"""Information about a metric."""
    checkouts: MetricTypedDict
    r"""Information about a metric."""
    succeeded_checkouts: MetricTypedDict
    r"""Information about a metric."""
    checkouts_conversion: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_customer_service: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_low_quality: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_missing_features: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_switched_service: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_too_complex: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_too_expensive: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_unused: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_other: MetricTypedDict
    r"""Information about a metric."""


class Metrics(BaseModel):
    orders: Metric
    r"""Information about a metric."""

    revenue: Metric
    r"""Information about a metric."""

    net_revenue: Metric
    r"""Information about a metric."""

    cumulative_revenue: Metric
    r"""Information about a metric."""

    net_cumulative_revenue: Metric
    r"""Information about a metric."""

    average_order_value: Metric
    r"""Information about a metric."""

    net_average_order_value: Metric
    r"""Information about a metric."""

    one_time_products: Metric
    r"""Information about a metric."""

    one_time_products_revenue: Metric
    r"""Information about a metric."""

    one_time_products_net_revenue: Metric
    r"""Information about a metric."""

    new_subscriptions: Metric
    r"""Information about a metric."""

    new_subscriptions_revenue: Metric
    r"""Information about a metric."""

    new_subscriptions_net_revenue: Metric
    r"""Information about a metric."""

    renewed_subscriptions: Metric
    r"""Information about a metric."""

    renewed_subscriptions_revenue: Metric
    r"""Information about a metric."""

    renewed_subscriptions_net_revenue: Metric
    r"""Information about a metric."""

    active_subscriptions: Metric
    r"""Information about a metric."""

    monthly_recurring_revenue: Metric
    r"""Information about a metric."""

    committed_monthly_recurring_revenue: Metric
    r"""Information about a metric."""

    checkouts: Metric
    r"""Information about a metric."""

    succeeded_checkouts: Metric
    r"""Information about a metric."""

    checkouts_conversion: Metric
    r"""Information about a metric."""

    canceled_subscriptions: Metric
    r"""Information about a metric."""

    canceled_subscriptions_customer_service: Metric
    r"""Information about a metric."""

    canceled_subscriptions_low_quality: Metric
    r"""Information about a metric."""

    canceled_subscriptions_missing_features: Metric
    r"""Information about a metric."""

    canceled_subscriptions_switched_service: Metric
    r"""Information about a metric."""

    canceled_subscriptions_too_complex: Metric
    r"""Information about a metric."""

    canceled_subscriptions_too_expensive: Metric
    r"""Information about a metric."""

    canceled_subscriptions_unused: Metric
    r"""Information about a metric."""

    canceled_subscriptions_other: Metric
    r"""Information about a metric."""
