"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookeventtype import WebhookEventType
from .webhookformat import WebhookFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class WebhookEndpointTypedDict(TypedDict):
    r"""A webhook endpoint."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    url: str
    r"""The URL where the webhook events will be sent."""
    format_: WebhookFormat
    secret: str
    r"""The secret used to sign the webhook events."""
    organization_id: str
    r"""The organization ID associated with the webhook endpoint."""
    events: List[WebhookEventType]
    r"""The events that will trigger the webhook."""


class WebhookEndpoint(BaseModel):
    r"""A webhook endpoint."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    url: str
    r"""The URL where the webhook events will be sent."""

    format_: Annotated[WebhookFormat, pydantic.Field(alias="format")]

    secret: str
    r"""The secret used to sign the webhook events."""

    organization_id: str
    r"""The organization ID associated with the webhook endpoint."""

    events: List[WebhookEventType]
    r"""The events that will trigger the webhook."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
